package com.hezongjian;

import java.io.*;

import android.content.Context;
import android.graphics.*;
import android.graphics.Bitmap.CompressFormat;


public class BitmapRender implements IGameRender {

	protected GameCore 	core;
	protected Context	cx;
	protected Bitmap[] cBmp;

	
	public BitmapRender(Context c)
	{
		this.core = GameCore.getInstance();
		this.cx = c;
		
		// allocate bitmap storage
		cBmp = new Bitmap[core.getKind()];
		
		// first, try to load from internal file
		// if failed, load from resource bitmap.
		if ( ! this.LoadBmpFromInternal())
		{
			this.LoadBmpFromRes();
		}
	}
	
	// load display bitmaps from Internal storage
	// return value, if enough image has been loaded.
	private boolean LoadBmpFromInternal()
	{
		boolean enough = true;
		String[] files = this.cx.fileList();

        if (files.length == 0)
            return false;
		
		// begin loading
		for (int i = 0; i < core.getKind(); i++)
		{
			String path = this.cx.getFilesDir().getAbsolutePath();
			if (! path.endsWith("/")) {
				path += '/';
			}
			String fullpath = path + files[i];
			
			cBmp[i] = BitmapFactory.decodeFile(fullpath);
			
			if (cBmp[i] == null)
				enough = false;
		}
		
		if (files.length < core.getKind())
			enough = false;
		
		return enough;
	}
	
	// load display bitmaps from Application's resource
	private boolean LoadBmpFromRes()
	{
		int BmpId[] = {
				R.drawable.sample_0,
				R.drawable.sample_1,
				R.drawable.sample_2,
				R.drawable.sample_3,
				R.drawable.sample_4,
				R.drawable.sample_5,
				R.drawable.sample_6,
				R.drawable.sample_7,
				R.drawable.sample_8,
				R.drawable.sample_9,
				R.drawable.sample_10
		};
		
		for (int i = 0; i < core.getKind(); i++)
		{
			if (cBmp[i] == null) {
				cBmp[i] = BitmapFactory.decodeResource(this.cx.getResources(), BmpId[i]);
			}
		}
		
		// save file to internal storage, use PNG format
		String[] files = this.cx.fileList();
		if (files.length == 0){
			this.SaveBmptoInternal();
		}
		
		return true;
	}
	
	// save display bitmap to internal storage
	private boolean SaveBmptoInternal()
	{
		for (int i = 0; i < core.getKind(); i++) {
			
			String FileName = String.format("%d.png", i);
			
			FileOutputStream fos;
			try {
				fos = this.cx.openFileOutput(FileName, Context.MODE_PRIVATE);

				cBmp[i].compress(CompressFormat.PNG, 100, fos);
			
				fos.close();
			} catch (Exception e) {
				e.printStackTrace();
				return false;
			}
		}
		return true;
	}
	
	
	public void Notify(int notifyType, Object pData) {
		
	}
	
	
	public void Render(Canvas c) {
		
		Paint p = new Paint();

		// screen width and height
		int w = c.getWidth();
		int h = c.getHeight();

        int elemW = w / core.getColCount();
        int elemH = h / core.getRowCount();

        // draw the picture row by row
        for (int i = 1; i <= core.getRowCount(); i++) {
            for (int j = 1; j <= core.getColCount(); j++) {

                if (core.getGrid()[i][j] == 0)	// needn't draw
                    continue;

                // get the drawing bitmap
                Bitmap bmp = this.cBmp[core.getGrid()[i][j] - 1];
                // get the drawing rectangle
                Rect rc = new Rect(elemW * (j - 1), elemH * (i - 1), elemW * j, elemH * i);

                c.drawBitmap(bmp, null, rc, p);
            }
        }

        // draw current selection
        if (core.getCurSel() != null) {
            p.setColor(Color.YELLOW);
            p.setStyle(Paint.Style.STROKE);
            p.setStrokeWidth(4);
            Rect rc = new Rect(elemW * (core.getCurSel().y - 1), elemH * (core.getCurSel().x - 1), elemW
                    * core.getCurSel().y, elemH * core.getCurSel().x);
            c.drawRect(rc, p);
        }


	} // end method

} // end class
