package com.hezongjian;

import java.util.*;
import android.content.Context;
import android.graphics.*;
import android.util.Log;

public class GameCore {
	private static GameCore pInst = null;

	// constant
	private int rowCount = 8;
	private int colCount = 5;
	private final int nKind = 10;
	private int orientation = 1;	// by default, portrait

	// game grid number meanings
	// 0 : empty
	// 1-nKind : kinds of grid
	// the grid is defined two row/col wider than the origin data.
	// it is used for easy path finding.
	private int grid[][] = new int[rowCount + 2][colCount + 2];

	
	// default using color renderer
	private IGameRender pRenderer;

	// current selection;
	Point curSel;

	GameCore() {
		
		//pRenderer = null;

		this.initGrid();
	}

	// print the grid, debug purpose only
	public void printGrid()
	{
		for(int i = 0; i < this.rowCount + 2; i++)
		{
			StringBuilder sb = new StringBuilder();
			for(int j = 0; j < this.colCount + 2; j++)
			{
				sb.append(this.grid[i][j]);
				sb.append('\t');
			}
			Log.v("LLL", sb.toString());
		}
	}
	
	public void restart()
	{
		this.initGrid();
	}
	
	// use random number to fill the grid;
	private void initGrid() {
		// create all the elements in the grid;
		ArrayList<Integer> elems = new ArrayList<Integer>();
		for (int i = 0; i < rowCount * colCount / 2; i++) {
			elems.add(i % nKind + 1);
			elems.add(i % nKind + 1);
		}

		// generate random number
		Random rd = new Random();

		// assign element
		for (int i = 0; i < rowCount + 2; i++) 
		{
			for (int j = 0; j < colCount + 2; j++) 
			{
				if (i == 0 || j == 0 || i == rowCount+1 || j == colCount+1)
					grid[i][j] = 0;
				else
				{
					int n = rd.nextInt(elems.size());
					grid[i][j] = elems.get(n);
					elems.remove(n);
				}
			}
		}
	}
	
	public void shuffle()
	{
		// use list to store the remaining elements
		List<Integer> al = new ArrayList<Integer>();
		
		// store the elements
		for(int i = 1; i <= this.rowCount; i++)
		{
			for (int j = 1; j <= this.colCount; j++)
			{
				if (this.grid[i][j] != 0)
				{
					al.add(this.grid[i][j]);
				}
			}
		}
		
		// generate random number
		Random rd = new Random();
		
		// put the elemtns
		for(int i = 1; i <= this.rowCount; i++)
		{
			for (int j = 1; j <= this.colCount; j++)
			{
				if (this.grid[i][j] != 0)
				{
					int idx = rd.nextInt(al.size());
					this.grid[i][j] = al.get(idx);
					al.remove(idx);
				}
			}
		}		
		
	}


	// Test whether two images can be linked together
	public boolean linkable(Point a, Point b) 
	{
		if (grid[a.x][a.y] != grid[b.x][b.y])
			return false;
		
		if (a.x == b.x && horizon(a, b))
			return true;
		if (a.y == b.y && vertical(a, b))
			return true;
		if (oneCorner(a, b))
			return true;
		else
			return twoCorner(a, b);
	}

	private boolean horizon(Point a, Point b) {
		if (a.x == b.x && a.y == b.y)
			return false;
		int x_start = a.y <= b.y ? a.y : b.y;
		int x_end = a.y <= b.y ? b.y : a.y;
		for (int x = x_start + 1; x < x_end; x++)
			if (grid[a.x][x] != 0) {
				return false;
			}
		return true;
	}

	private boolean vertical(Point a, Point b) {
		if (a.x == b.x && a.y == b.y)
			return false;
		int y_start = a.x <= b.x ? a.x : b.x;
		int y_end = a.x <= b.x ? b.x : a.x;
		for (int y = y_start + 1; y < y_end; y++)
			if (grid[y][a.y] != 0)
				return false;

		return true;
	}

	// check if two points can be linked via one corner
	// if OK, return the corner point
	private boolean oneCorner(Point a, Point b) {
		Point c = new Point(a.x, b.y);
		Point d = new Point(b.x, a.y);
		if (grid[c.x][c.y] == 0) 
		{
			if (horizon(a, c) && vertical(b, c))
			{
				return true;
			}
		}
		if (grid[d.x][d.y] == 0) {
			if (vertical(a, d) && horizon(b, d))
			{
				return true;
			}
		} 

		return false;
	}
	
	class Line   
    {   
        public Point a;   
        public Point b;   
        public int direct;   
  
        public Line()   
        {   
        }   
  
        public Line(int direct, Point a, Point b)   
        {   
            this.direct = direct;   
            this.a = a;   
            this.b = b;   
        }   
    }  
	
	private boolean twoCorner(Point a, Point b)   
	{   
		
		LinkedList<Line> li = new LinkedList<Line>();  
        for(int y = a.y; y >= 0; y--)   
            if(grid[a.x][y] == 0 && grid[b.x][y] == 0 && vertical(new Point(a.x, y), new Point(b.x, y)))   
            	li.add(new Line(0, new Point(a.x, y), new Point(b.x, y)));   
  
        for(int y = a.y; y <= colCount + 1; y++)   
            if(grid[a.x][y] ==0 && grid[b.x][y] ==0 && vertical(new Point(a.x, y), new Point(b.x, y)))   
                li.add(new Line(0, new Point(a.x, y), new Point(b.x, y)));   
  
        for(int x = a.x; x >= 0; x--)   
            if(grid[x][a.y] == 0 && grid[x][b.y] ==0 && horizon(new Point(x, a.y), new Point(x, b.y)))   
                li.add(new Line(1, new Point(x, a.y), new Point(x, b.y)));   
  
        for(int x = a.x; x <= rowCount + 1; x++)   
            if(grid[x][a.y] == 0 && grid[x][b.y] == 0 && horizon(new Point(x, a.y), new Point(x, b.y)))   
                li.add(new Line(1, new Point(x, a.y), new Point(x, b.y)));   
  
	    if(li.isEmpty())   
	        return false;   
	    for(int index = 0; index < li.size(); index++)
	    {   
	        Line line = li.get(index);   
	        if(line.direct == 1)
	        {   
	            if(vertical(a, line.a) && vertical(b, line.b))
	            {   
	                return true;   
	            }     
	        } 
	        else if(horizon(a, line.a) && horizon(b, line.b))
	        {   
	            return true;   
	        }   
	    }   
	    return false;   
	}  
	
	boolean isVictory()
	{
		for(int i = 1; i <= rowCount; i++)
		{
			for(int j = 1; j <= colCount; j++)
			{
				if (grid[i][j] != 0)
					return false;
			}
		}
		return true;
	}
	
	// this is cell phone multi-touch version
	public int select(Point x, Point y)
	{
		// clear current selection
		if (this.curSel != null)
		{
			this.curSel = null;
		}
		
		select(x.x, x.y);
		select(y.x, y.y);
		
		return -1;
	}

	// return value indicates: 
	//	-1	:	invalid selection
	//	0	:	select an item
	//	1	:	game complete
	public int select(int x, int y) {
		// invalid
		if (x < 0 || y < 0 || x > rowCount + 1 || y > colCount + 1)
			return -1;

		// already cleared
		if (grid[x][y] == 0)
			return -1;

		// if no selection ,select one
		if (this.curSel == null) {
			this.curSel = new Point(x, y);
			return 0;
		}

		// try to find the link path
		boolean result = this.linkable(this.curSel, new Point(x, y));

		// if no path
		if (result == true)
		{
			// clear both two
			grid[curSel.x][curSel.y] = 0;
			grid[x][y] = 0;
		}
		// de-select current selection
		this.curSel = null;
		
		if (this.isVictory())
			return 1;
		else
			return 0;

	}

	public void Render(Canvas c) {

		if (this.pRenderer == null)
			return;
		
		this.pRenderer.Render(c);

	}
	
	public void setRender(IGameRender r)
	{
		this.pRenderer = r;
	}

    public void setRender(Context cx, int type) {

        if (type == 1) {
            GameCore.getInstance().setRender(new ColorRender());
            return ;
        }

        PhotoCollector pc = new PhotoCollector(cx);

        switch (type) {
            case 3: // contact
                pc.loadFromContactPhoto();
                break;
            case 4: // application icon
                pc.loadFromApplicationIcon();
                break;
            case 5: // google
                break;
            default:
                break;
        }

        pc.savePhotoToInternal();
        setRender(new BitmapRender(cx));

    }

	// singleton
	public static GameCore getInstance() {
		if (pInst == null) {
			pInst = new GameCore();
		}

		return pInst;
	}

	public int getRowCount() {
		return rowCount;
	}

	public int getColCount() {
		return colCount;
	}


	public int getKind() {
		return nKind;
	}


	public int[][] getGrid() {
		return grid;
	}


	public Point getCurSel() {
		return curSel;
	}



	// use this method to rotate the screen
	// orientation value:
	// 0 : SCREEN_ORIENTATION_LANDSCAPE
	// 1 : SCREEN_ORIENTATION_PORTRAIT
	// return value : the original orientation.
	public int setOrientation(int newOret) {
		
		// firstly, check if we really need to rotate screen.
		if (newOret == this.orientation)
			return this.orientation;
		
		// we're ready to rotate the screen.
		// reallocate the grid array
		
		int newcolCount = this.rowCount;
		int newrowCount = this.colCount;
		
		// secondly, reallocate the grid array
		int newgrid[][] = new int[newrowCount + 2][newcolCount + 2];
		
		// copy the old data into the new one, the map is
		// y-> newx
		// oldrow-x -> newy
		for(int i = 1; i < newrowCount + 1; i++)
		{
			for(int j = 1; j < newcolCount + 1; j++)
			{
				newgrid[i][j] = grid[j][this.colCount + 1 - i];
			}
		}
		
		// replace it
		this.grid = newgrid;
		
		// swap old and new row/col count
		this.rowCount = newrowCount;
		this.colCount = newcolCount;
		
		// change the orientation
		this.orientation = newOret;
		
		return newOret;
	}


	public int getOrientation() {
		return orientation;
	}

}
