package com.hezongjian;

import android.content.*;
import android.graphics.*;
import android.util.*;
import android.view.*;
import android.view.animation.*;
import android.widget.Toast;
import android.hardware.*;
import android.preference.*;

public class GameView extends View implements View.OnTouchListener, android.hardware.SensorEventListener{

	private int canvasWidth;
	private int canvasHeight;


	public GameView(Context context, AttributeSet attr) {
		super(context, attr);
		this.Init(context);
	}

	public GameView(Context context, AttributeSet attrs, int defStyle) {
		super(context, attrs, defStyle);
		this.Init(context);
	}



	
	private void Init(Context cx)
	{
		// set renderer based on preference
        SharedPreferences sp = PreferenceManager.getDefaultSharedPreferences(cx);
        int type = Integer.parseInt(sp.getString("game_type", "1"));

        GameCore.getInstance().setRender(new BitmapRender(cx));

		this.setOnTouchListener(this);
		
		this.invalidate();
	}
	
	
	@Override
	protected void onDraw(Canvas canvas) {
		//super.onDraw(canvas);

		GameCore.getInstance().Render(canvas);

	}	

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.view.View#onSizeChanged(int, int, int, int)
	 */
	@Override
	protected void onSizeChanged(int w, int h, int oldw, int oldh) {
		//super.onSizeChanged(w, h, oldw, oldh);
		
		// NOTE: handle rotation method here.
		this.canvasHeight = h;
		this.canvasWidth = w;
		
		if (h > w)	// portrait
		{
			GameCore.getInstance().setOrientation(1);
		}
		else // landscape
		{
			GameCore.getInstance().setOrientation(0);
		}
		
	}

	
	// this is the touch event
	// first, check the touch count
	// then, invoke relative touch version
	public boolean onTouch(View v, MotionEvent event) {
		int nPtCnt = event.getPointerCount();
		int r = 0; // game result
		
		int numRow, numCol;

		numRow = GameCore.getInstance().getRowCount();
		numCol = GameCore.getInstance().getColCount();

		if (nPtCnt == 1 && event.getAction() == MotionEvent.ACTION_DOWN) // single touch
		{

			float ptX = event.getX();
			float ptY = event.getY();

			// calculate current point;
			int x = (int) (ptY / (this.canvasHeight / numRow));
			int y = (int) (ptX / (this.canvasWidth / numCol));

			x++;
			y++;

			r = GameCore.getInstance().select(x, y);
		} 
		else if (nPtCnt == 2 && event.getAction() == MotionEvent.ACTION_POINTER_2_DOWN) // multi-touch
		{
			Log.i("LLL", "MultiTouched");
			int y1 = (int) (event.getX(0) / (this.canvasWidth / numCol));
			int y2 = (int) (event.getX(1) / (this.canvasWidth / numCol));
			
			int x1 = (int) (event.getY(0) / (this.canvasHeight / numRow));
			int x2 = (int) (event.getY(1) / (this.canvasHeight / numRow));
			
			x1++;
			x2++;
			y1++;
			y2++;
			
			r = GameCore.getInstance().select(new Point(x1,y1), new Point(x2, y2));
		}
		
		// refresh
		this.invalidate();

		// game complete
		if (r == 1) {
			Toast.makeText(this.getContext(),
					"Game Complete, Let's Play another round",
					Toast.LENGTH_LONG).show();
			GameCore.getInstance().restart();
		}

		return true;
	}

	// do nothing here
	public void onAccuracyChanged(Sensor sensor, int accuracy) {
		// TODO Auto-generated method stub
		
	}

	// used to shuffle the grid.
	public void onSensorChanged(SensorEvent event) {
		
		// ignore all other sensors
		if (event.sensor.getType() != Sensor.TYPE_ACCELEROMETER)
		{
			return ;
		}
		
		float [] v = event.values;
		
		float aX = Math.abs(v[0]);
		float aY = Math.abs(v[1]);
	
		if (aX > 15 || aY > 15) // a must be greater than 15.
		{
			View view = findViewById(R.id.GameView);
			GameCore.getInstance().shuffle();
			
			// refresh
			view.invalidate();
			Toast.makeText(this.getContext(), "Game Shuffled", Toast.LENGTH_SHORT).show();
			
			// animate the screen
			Animation a = new TranslateAnimation(-aX, aX, -aY, aY);
			a.setRepeatCount(5);
			a.setRepeatMode(Animation.RESTART);
			a.setDuration(100);
			
			view.startAnimation(a);
			
		}
				
	}

}
