package com.hezongjian;

import android.content.ContentResolver;
import android.content.ContentUris;
import android.content.Context;
import android.content.Intent;
import android.content.pm.ResolveInfo;
import android.database.Cursor;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.PixelFormat;
import android.graphics.drawable.Drawable;
import android.net.Uri;
import android.provider.BaseColumns;
import android.provider.ContactsContract;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.*;
import java.net.URL;
import java.net.URLConnection;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * Created by hezongjian on 5/28/13.
 */
public class PhotoCollector {

    public PhotoCollector(Context cx) {
        this.mCx = cx;
    }

    // internal class, used to store bitmap and its description
    public class BitmapPair
    {
        public BitmapPair()
        {

        }
        public BitmapPair(Bitmap b, String d)
        {
            Bmp = b;
            Desc = d;
        }
        public Bitmap Bmp;
        public String Desc;
    }

    // the collected photos.
    private ArrayList<BitmapPair> mPhotos = new ArrayList<BitmapPair>();

    // the collected number
    private int mItemCount = 20;

    private Context mCx;

    public ArrayList<BitmapPair> getPhoto() {

        // if less than mItemCount, should add default picture.
        return this.mPhotos;
    }

    public void savePhotoToInternal() {

        for (int i = 0; i < mPhotos.size(); i++) {

            String FileName = String.format("%d.png", i);

            FileOutputStream fos;
            try {
                fos = this.mCx.openFileOutput(FileName, Context.MODE_PRIVATE);

                mPhotos.get(i).Bmp.compress(Bitmap.CompressFormat.PNG, 100, fos);

                fos.close();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    public void loadFromContactPhoto()
    {
        // clear dock
        this.mPhotos.clear();

        // find ALL id and user name
        ContentResolver cr = this.mCx.getContentResolver();

        Uri uri = android.provider.ContactsContract.Contacts.CONTENT_URI;
        String prj[] =
                {
                        BaseColumns._ID,
                        android.provider.ContactsContract.Contacts.DISPLAY_NAME
                };

        String sel = String.format("%s is not null", android.provider.ContactsContract.Contacts.PHOTO_ID);

        Cursor cur = cr.query(uri, prj, sel, null, null);

        if (cur == null)
            return ;

        long 	contactId;
        String 	contactName;

        cur.moveToFirst();
        while(! cur.isLast())
        {
            contactId = cur.getLong(0);
            contactName = cur.getString(1);

            Uri contactPhotoUri = ContentUris.withAppendedId(ContactsContract.Contacts.CONTENT_URI, contactId);

            InputStream photoDataStream = ContactsContract.Contacts.openContactPhotoInputStream(cr, contactPhotoUri);
            Bitmap photo = BitmapFactory.decodeStream(photoDataStream);

            if (photo != null) {
                mPhotos.add(new BitmapPair(photo, contactName));
            }

            cur.moveToNext();
        }

        cur.close();

    }


    // load all the application's icons that installed.
    public void loadFromApplicationIcon()
    {
        this.mPhotos.clear();

        // displayed in app lists
        Intent mainIntent = new Intent(Intent.ACTION_MAIN, null);
        mainIntent.addCategory(Intent.CATEGORY_LAUNCHER);

        List<ResolveInfo> mApps = this.mCx.getPackageManager().queryIntentActivities(mainIntent, 0);

        Iterator<ResolveInfo> it = mApps.iterator();

        while(it.hasNext())
        {
            ResolveInfo info = it.next();
            // get the display icon.
            Drawable d = info.activityInfo.loadIcon(this.mCx.getPackageManager());
            // get the display name.
            String appName = info.loadLabel(this.mCx.getPackageManager()).toString();

            this.mPhotos.add(new BitmapPair(this.drawableToBitmap(d), appName));
        }
    }

    public void loadFromGoogle(String input) {

        URL url = null;
        URLConnection connection;
        JSONObject json = null;

		/* the google service URL
		 * url parameters description:
		 *  v=1.0  			default
		 *  imgsz=icon		only search icon image
		 *  rsz=8			return 8 results once
		 *  q=				query content
		 */
        String googleService = String.format(
                "http://ajax.googleapis.com/ajax/services/search/images?&v=1.0&imgsz=icon&rsz=%d&q=%s",
                this.mItemCount,
                input);

        try {
            url = new URL(googleService);
            connection = url.openConnection();

            String line;
            StringBuilder builder = new StringBuilder();
            BufferedReader reader = new BufferedReader(new InputStreamReader(
                    connection.getInputStream()));
            while ((line = reader.readLine()) != null) {
                builder.append(line);
            }

            json = new JSONObject(builder.toString());

        } catch (IOException e) {
            e.printStackTrace();
        } catch (JSONException e) {
            e.printStackTrace();
        }

        // now have some fun with the results...

        // get the results array
        JSONArray results = null;
        try {
            // get the search result according to google's format specification
            // the spec is at : http://code.google.com/apis/ajaxsearch/documentation/reference.html
            results = json.getJSONObject("responseData").getJSONArray("results");

            // for each result get the image and it's title
            for(int i = 0; i < results.length(); i++) {
                // get the result object
                JSONObject result = results.getJSONObject(i);

                // get the title
                String title = result.getString("titleNoFormatting");
                String imgUrl = result.getString("url");

                // get the image from Internet.
                Bitmap bmp = null;
                try{
                    bmp = this.addressToBitmap(imgUrl);
                }catch(Exception e) {
                    e.printStackTrace();
                }

                // add to display
                if (bmp != null) {
                    this.mPhotos.add(new BitmapPair(bmp, title));
                }

            }
        } catch (org.json.JSONException e){
            e.printStackTrace();
        }
    }

    // convert an URL to a bitmap, copied from Internet.
    private static Bitmap addressToBitmap(String address) throws Exception{
        URL url = new URL(address);
        URLConnection conn = url.openConnection();
        conn.connect();
        InputStream in = conn.getInputStream();
        BufferedInputStream bis= new BufferedInputStream(in);

        Bitmap bitmap = BitmapFactory.decodeStream(bis);

        return bitmap;
    }

    // convert a drawable to a bitmap, copied from internet.
    private static Bitmap drawableToBitmap(Drawable drawable) {

        Bitmap bitmap = Bitmap
                .createBitmap(
                        drawable.getIntrinsicWidth(),
                        drawable.getIntrinsicHeight(),
                        drawable.getOpacity() != PixelFormat.OPAQUE ? Bitmap.Config.ARGB_8888
                                : Bitmap.Config.RGB_565);
        Canvas canvas = new Canvas(bitmap);
        drawable.setBounds(0, 0, drawable.getIntrinsicWidth(), drawable.getIntrinsicHeight());
        drawable.draw(canvas);
        return bitmap;
    }
}
